/**
 * Sparkler
 *
 * Michal Galan
 *
 * MOUSE
 * drag                : draw with text
 * 
 * KEYS
 * del, backspace      : clear screen
 * s                   : save png
 * r                   : start pdf recording
 * e                   : stop pdf recording
 */

import processing.pdf.*;
import java.util.Calendar;

boolean recordPDF = false;

float x = 0, y = 0;
float stepSize = 500.0;

int stepSizeMin = 3;
float textAngleDistortion = 0.3;
float fractalAngleDistortion = 90;

int mode = 0;

color white = color(255,255,255);
color black = color(0,0,0);
color colLineHappy = color(254, 220, 1);
color colLineDepresive = color(89, 88, 93);

ArrayList<Sparkler> sparkles = new ArrayList(10);
ArrayList<PVector> stars = new ArrayList(30);
HashMap<String,PVector> starPositions = new HashMap(30);

int starBlinking = 10;

float textY = 0;
float minTextY = 130;
float textZ = -200;

PFont font;
PFont fontIntro;

String text = "It is a dark time of  semester.\nUniversity     exams     started\nappearing,   the   earliest   are\ndestroyed now.  Students  are\nfully engaged in Universal war\non projects,  leaving no space\nfor   rest.  During one battle, a\npowerful  weapon  of life  was\nconstructed.                              \n\nTHE LIFE SPARKLER.              ";
String textIntro = "A long time ago in a galaxy\nfar, far away....";
void setup() {
  font = createFont("FRABKIT.TTF",50);
  fontIntro = createFont("Arial Bold",50);
  textFont(font, 30);
  textMode(MODEL);
  
  for(int i=0; i < 1800; i++){
    float x = random(width);
    float y = random(height);
    PVector p = new PVector(x,y,1);
    stars.add(p);
    starPositions.put(int(x)+" "+int(y),p);
  }
  
  size(displayWidth, displayHeight,P3D);
  //frameRate(5);
  //size(800, 600, P3D);
  textY = height;
  background(white);
  smooth();
  cursor(CROSS);

  x = width/2;
  y = height/2;

  fill(black);
}

void fade(){
  if (mode == 0){
    fill(black,5);
    rect(0,0,width,height);
    fill(colLineHappy);
    stroke(colLineHappy);
  }
  if (mode == 1){
    fill(white,5);
    rect(0,0,width,height);
    fill(colLineDepresive);
    stroke(colLineDepresive);
  }
}

void drawStars(boolean blink){
  if (mode == 0){
    fill(colLineHappy);
    stroke(colLineHappy);
  }
  if (mode == 1){
    fill(colLineDepresive);
    stroke(colLineDepresive);
  }
  for (PVector p : stars) {
    if (p.z == 0) continue;
    if (!blink || int(random(starBlinking)) == 0){
      ellipse(p.x, p.y, 1, 1);
    }
  }
}

void draw() {

  if (textY > minTextY-(768-height)){
    background(0);
    drawStars(false);
    if (textY > height-80){
      textAlign(LEFT);
      fill(14, 179, 218);
      textFont(fontIntro, 30);
      text(textIntro, width/2-textWidth(textIntro)/2, height/4);
    }
    
    fill(248, 245, 58);
    rotateX(PI/3);
    textAlign(CENTER);
    text(text, width/2, textY, textZ);
    textY-=0.5;
    return;
  }else if(int(textY) == int(minTextY-(768-height))){
    background(white);
    textY = minTextY-9999999;
  }

  //for (int i = 0; i < 30; i++) {
  // ellipse(random(width), random(height), 1, 1);
  //}
  drawStars(true);
  fade();
  for(Sparkler sparkle : sparkles){
    sparkle.drawSparkle();
    sparkle.moveRandom(starPositions);
  }

  if (mousePressed) {
    float d = dist(x,y, mouseX,mouseY);
    stepSize = stepSizeMin+d/2;
    if (d > stepSize) {
      float angle = atan2(mouseY-y, mouseX-x);
      x = x + cos(angle) * stepSize;
      y = y + sin(angle) * stepSize;
    }
  }
  //saveFrame("frames/####.tga");
}


void mousePressed() {
  sparkles.add(new Sparkler(mouseX,mouseY,50,1000));
  x = mouseX;
  y = mouseY;
}

void keyReleased() {
  if (key == 's' || key == 'S') saveFrame(timestamp()+"_##.png");
  if (key == DELETE || key == BACKSPACE) background(255);
  
  if (key == '1'){
    mode = 0;
    fill(white);
    background(white);
  }
  if (key == '2'){
    mode = 1;
    fill(black);
    background(black);
  }
  
  if (key =='p' || key =='P') {
    save("capture.png");
  }

  // ------ pdf export ------
  // press 'r' to start pdf recordPDF and 'e' to stop it
  // ONLY by pressing 'e' the pdf is saved to disk!
  if (key =='r' || key =='R') {
    if (recordPDF == false) {
      beginRecord(PDF, timestamp()+".pdf");
      println("recording started");
      recordPDF = true;
      textAlign(LEFT);
      fill(0);
    }
  } 
  else if (key == 'e' || key =='E') {
    if (recordPDF) {
      println("recording stopped");
      endRecord();
      recordPDF = false;
      background(255); 
    }
  } 
}

void keyPressed() {

}

// timestamp
String timestamp() {
  Calendar now = Calendar.getInstance();
  return String.format("%1$ty%1$tm%1$td_%1$tH%1$tM%1$tS", now);
}